############################################################################
# Testing of Unimodality
# example R code belonging to
# Jamil, T & ter Braak C.J.F (2013) Generalized Linear Mixed Models can
# detect unimodal species-environment relationships. PeerJ.
# Creative Commons Attribution license 3.0
###########################################################################
  rm(list=ls(all=TRUE))
  library(vegan) # for Dune data and for comparison with length of gradient in DCA
  library(lme4)  # for the GLMM
############################################################################
# function defininiton (begin): Test.Graph.unimodal 
############################################################################
  #source("Test.Graph.unimodal.R") # you can also store the function as Test.Graph.unimodal.R and load it by using source
Test.Graph.unimodal<-function(y,x,species,site, family = binomial,  xlab = "X", verbose = TRUE, display.plot = TRUE ) {
  # test unimodality as by equations (1) and  (5) of  Jamil and ter Braak 2013 Peer J paper
  # make the graph visualizing the unimodality
  # y = vector with inflated 1/0 data (family = binomial) or
  # y = vector with inflated abundance (family = Poisson and, likely, add exta term to lmer model for overdispersion,
  # x = vector with inflated environmental variable data
  # species = vector or factor coding for species
  # sites = vector or factor coding of sites
  #        see  http://onlinelibrary.wiley.com/doi/10.1111/j.1654-1103.2012.12036.x/suppinfo
  #        and see example
  # value: 
  # test       anova test on quadratic effect
  # summary    summary of the regression on x, x^2 and S (species total)
  # lmer.model lmer object of the GLMM model 
  # beta       beta_j in equation (1) in Jamil and ter Braak 2013 Peer J paper
  # site.data  data frame of data displayed in the plot (site effects gamma and x)
  #
  species <- factor(species)
  site <- factor(site)
  require(lme4)
  # implementation of equation (1) in Jamil and ter Braak 2013 PeerJ.
  M1 <- lmer (y ~ x  +(1 + x | species)+(1|site),family=family)
  site.ef<-as.vector(ranef(M1)$site )
  X<-as.vector(unlist( by(x,site, mean))) 
  data<-data.frame(X=X,site.ef)
  colnames(data)<-c("X" ,"site.ef")
  SP<-unlist(by(y,site, sum)) 
  Sp.dat<- matrix(SP,ncol=1) 
  Dat<-data.frame(site.ef,Sp.dat, X)
  colnames(Dat)<-c("site.effect" ,"Species.sum","X" )
  lm1 <- lm( site.effect~X+Species.sum,Dat)
  lm2 <- lm( site.effect~poly(X,2)+Species.sum,Dat)
  summary.uni<-summary(lm2)
  anova.uni <- anova(lm1,lm2)
  if (verbose){
    print("Statistical test on unimodality ") 
    print(anova.uni)
    print(summary.uni)
  }
  par(bty="n") 
  if (display.plot) plot(data$X,data$site.ef,pch=16,xlab=xlab,ylab="Site effects")
  # beta_j in equation (1) in Jamil and ter Braak 2013 Peer J paper
  beta.M1 <- ranef(M1)$species[,"x"]
  names(beta.M1) <- levels(species)
  list(test= anova.uni, summary = summary.uni, lmer.model = M1, beta = beta.M1 ,site.data = Dat)
} 
############################################################################
# function defininiton (end)
############################################################################

############################################################################
# Dune Meadows data example
############################################################################
  data(dune)
# expand community data matrix dune to a vector (vectorize)
  sitespec <- expand.grid(rownames(dune),colnames(dune))
  site <-sitespec[,1]; species<-sitespec[,2]
  abun <- as.vector(as.matrix(dune))
  y <- 1*(abun>0)  # transformation to presence - absence
  data(dune.env)
# make environmental variable Moisture in dune.env  quantitative as in Jongman et al. 1995
  Moisture <- as.numeric(levels(dune.env$Moisture))[dune.env$Moisture]
# expand Moisture to x 
  x <- rep(Moisture, ncol(dune))
# test unimodality as by equation (5) of  Jamil and ter Braak 2013 Peer J paper
# make the graph visualizing the unimodality
  tg <- Test.Graph.unimodal(y,x,species,site, xlab =  "Moisture")

############################################################################
# compare with length of gradient in detrended correspondence analysis
############################################################################
  dune1 <- as.data.frame(1*(dune>0)) 
  decorana(dune1)
  
############################################################################
# compare random effects regression coefficients from glmm with optima from 
# separate fits using glm. See Figure 3
############################################################################

############################################################################
# function defininiton:
############################################################################
 #source("se.coef.R") 
 se.coef<-
  function(object)
  {
    object.class <- class(object)[[1]]
    sqrt (diag(vcov(object)))
  }
############################################################################
# function defininiton:
############################################################################

  #comparison of beta 
  m <- ncol(dune); species.names <- colnames(dune)
  coeff<-matrix(0,m,3)
  se.coeff<-matrix(0,m,3)
  for (j in 1:m){ 
              sp <- species.names[j]
              lm1 <-glm(y[species %in% sp]~x[species==sp]+I(x[species==sp]^2),family=binomial(link="logit"))
              coeff[j,]<-coef(lm1) 
              se.coeff[j,] <- se.coef(lm1)
              }
  b0<-coeff[,1] 
  b1<-coeff[,2]                 
  b2<-coeff[,3]   
  se.b2<-se.coeff[,3]
  opt<-(- b1) /(2* b2)   
  tratio<-b2/se.b2
  beta.M1 <- tg$beta
  beta1<- data.frame(beta.M1, opt,b1,b2,tratio)
  beta1<-subset(beta1, beta1$b2<0)
  par(bty="n") 
  plot(beta1$opt,beta1$beta.M1,pch=16,xlab=quote(u [j]), ylab=quote(beta[j]))

